package ch.epfl.collections;

import java.util.ArrayList;
import java.util.Collections;

import org.junit.Test;
import static org.junit.Assert.*;

/**
 * Abstract tests for our Set implementation. To test a new implementation
 * you should implement a new child class which provides a Set interface.
 * For example:
 * 
 *  @see TestTreeSet
 *
 */
abstract public class TestSet {
    abstract Set<Integer> newEmptyIntSet();

    @Test
    public void testIsEmpty() {
        Set<Integer> s = newEmptyIntSet();
        assertTrue(s.isEmpty());
        s.add(1);
        assertFalse(s.isEmpty());
        s.remove(1);
        assertTrue(s.isEmpty());
    }

    @Test
    public void testSize() {
        Set<Integer> s = newEmptyIntSet();
        for (int i = 0; i < 20; ++i) {
            assertEquals(i, s.size());
            s.add(i);
        }
    }

    @Test
    public void testRemove() {
        Set<Integer> s = newEmptyIntSet();
        int[] elems = new int[] { 3, 1, 5, 7, 9, 15, 13, 11 };
        for (int e: elems)
            s.add(e);
        for (int e: elems) {
            assertTrue(s.contains(e));
            s.remove(e);
            assertFalse(s.contains(e));
        }
        assertTrue(s.isEmpty());
    }

    @Test
    public void testContains() {
        Set<Integer> s = newEmptyIntSet();
        assertFalse(s.contains(1));
        s.add(1);
        assertTrue(s.contains(1));
        s.add(0);
        s.add(2);
        assertFalse(s.contains(-1));
        assertTrue(s.contains(0));
        assertTrue(s.contains(1));
        assertTrue(s.contains(2));
        assertFalse(s.contains(3));
    }
    
    @Test
    public void testIterator() {
        Set<Integer> s = newEmptyIntSet();
        ArrayList<Integer> l = new ArrayList<Integer>();
        for (int i = 0; i < 32; ++i)
            l.add(i);
        Collections.shuffle(l);
        for (int i: l)
            s.add(i);
        int bitSet = 0;
        for (int i: s) {
            assertTrue(0 <= i && i < 32);
            final int mask = 1 << i;
            assertTrue((bitSet & mask) == 0);
            bitSet |= mask;
        }
        assertEquals(0xFFFFFFFF, bitSet);
    }
}
